package com.cogentlogic.training.android.crypto;

import java.security.Key;
import java.security.SecureRandom;
import java.security.Security;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.spec.IvParameterSpec;

import org.spongycastle.jce.provider.BouncyCastleProvider;

import android.app.Activity;
import android.os.Bundle;
import android.view.Menu;

public class MainActivity extends Activity
{
	static
	{
		Security.addProvider(new BouncyCastleProvider());
	}

	private final static String s_strMessage = "The quick brown fox jumped over the lazy dogs.";

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        try
        {
			SecureRandom sr = new SecureRandom();
			byte[] bytesIV = new byte[16];
			sr.nextBytes(bytesIV);
			IvParameterSpec iv = new IvParameterSpec(bytesIV);

			KeyGenerator kg = KeyGenerator.getInstance("AES", "SC");
			kg.init(256);															// key size
			Key key256AES = kg.generateKey();

			System.out.println("Key: " + Utilities.toHex(key256AES.getEncoded()));
			System.out.println("IV: " + Utilities.toHex(bytesIV));
			System.out.println("Plain text: " + s_strMessage);

			Cipher cipher = Cipher.getInstance("AES/CBC/PKCS7Padding", "SC");		// CBC requires an initialization vector

			// Encrypt
			byte[] byteMessage = s_strMessage.getBytes("UTF-8");
			cipher.init(Cipher.ENCRYPT_MODE, key256AES, iv);
			byte[] cipherText = new byte[cipher.getOutputSize(byteMessage.length)];
			int nCipherLen = cipher.update(byteMessage, 0, byteMessage.length, cipherText, 0);
			nCipherLen += cipher.doFinal(cipherText, nCipherLen);
			System.out.println("Cipher text: " + Utilities.toHex(cipherText) + "  --  " + nCipherLen + " bytes");

			// Decrypt
	        byte[] bytesDecryptedText = new byte[nCipherLen];
	        cipher.init(Cipher.DECRYPT_MODE, key256AES, iv);
	        int nDecryptedTextLen = cipher.update(cipherText, 0, nCipherLen, bytesDecryptedText, 0);
	        nDecryptedTextLen += cipher.doFinal(bytesDecryptedText, nDecryptedTextLen);
	        String strDecryptedText = new String(bytesDecryptedText, 0, nDecryptedTextLen, "UTF-8");
	        System.out.println("Decrypted text: " + strDecryptedText);
        }
        catch (Exception except)
        {
        	except.printStackTrace();
        }
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }
    
	public static class Utilities
	{
		private static final String s_strHexDigits = "0123456789ABCDEF";

		public static String toHex(byte[] dataIn)
		{
			StringBuffer sbOut = new StringBuffer();
			for (int n=0; n<dataIn.length; n++)
			{
				sbOut.append(s_strHexDigits.charAt((dataIn[n] >> 4) & 0xF));	// high nybble
				sbOut.append(s_strHexDigits.charAt(dataIn[n] & 0xF));			// low nybble
			}
			return sbOut.toString();
		}


		public static byte[] toData(String strIn)
		{
			int nLen = strIn.length();
			byte[] bytesOut = new byte[nLen / 2];		// two hex digits -> one byte
			for (int n=0; n<nLen; n+=2)
				bytesOut[n/2] = (byte)((Character.digit(strIn.charAt(n), 16) << 4) + Character.digit(strIn.charAt(n+1), 16));
			return bytesOut;
		}
	}
}
